# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTIBILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

bl_info = {
    "name" : "Voice Attack Blender Bridge",
    "author" : "Eric Medlen",
    "description" : "This addon acts as a bridge for Voice Attack to input Blender python commands via shortcut ctrl alt shift F11.",
    "blender" : (3, 0, 0),
    "version" : (0, 0, 3),
    "location" : "",
    "warning" : "",
    "category" : "Generic"
}

# Import you classes here so they can be accessed via VA python commands.
from . import set_origin,text_editor,snapping_3d,node_ops,object_data_ops,viewport_display

import bpy
import ctypes
import sys
import os

#dir = os.path.dirname(bpy.data.filepath)
#dir = os.path.dirname(os.path.realpath(__file__))
#if not dir in sys.path:
#    sys.path.append(dir)

# ctype stuff to access the windows clipboard.  Foreign language to me...
CF_TEXT = 1

kernel32 = ctypes.windll.kernel32
kernel32.GlobalLock.argtypes = [ctypes.c_void_p]
kernel32.GlobalLock.restype = ctypes.c_void_p
kernel32.GlobalUnlock.argtypes = [ctypes.c_void_p]
user32 = ctypes.windll.user32
user32.GetClipboardData.restype = ctypes.c_void_p

def get_clipboard_text():
    user32.OpenClipboard(0)
    try:
        if user32.IsClipboardFormatAvailable(CF_TEXT):
            data = user32.GetClipboardData(CF_TEXT)
            data_locked = kernel32.GlobalLock(data)
            text = ctypes.c_char_p(data_locked)
            valueRaw = text.value
            kernel32.GlobalUnlock(data_locked)
            
            myText=str(valueRaw)
            myText=myText.replace("b","",1)
            myText=myText[1:-1]#Sub string for end and beginning removal.
            return myText
    finally:
        user32.CloseClipboard()


class RunABlenderPythonCommandForVoiceAttack(bpy.types.Operator):
    """Run A Blender Python Command"""
    bl_idname = "pythonvoiceattack.code_text_dialog"
    bl_label = "Run A Blender Python Command"
    bl_property = "codeText"#Make it default selected option.
    bl_options={'REGISTER','UNDO'}
    
    codeText : bpy.props.StringProperty(
        default="Code Here", 
        )
    
    @classmethod
    def poll(cls, context):
        return True

    def execute(self, context):
        #Straight clipboard.
        self.codeText=get_clipboard_text()
        print ("Voice Attack command for Blender = "+self.codeText)
        exec (self.codeText)

#        #Tokenized
#        path=get_clipboard_text().split(">")
#        exec(f"from . import {path[0]}\n{path[0]}.{path[1]}()")
        return {'FINISHED'}

#    def invoke(self, context, event):
#        return context.window_manager.invoke_props_dialog(self, width=100)

#input panel
#    def draw(self, context):
#        row = self.layout
#        row.prop(self, "codeText", text="Code Text")
#        row.separator()

#never used.
#class OF_PT_Panel(bpy.types.Panel):
#    """Creates a Panel in the Object properties window"""
#    bl_idname = "OF_my_panel"
#    bl_label = "OF Tools"
#    bl_space_type = "VIEW_3D"
#    bl_region_type = "TOOLS"
#    #bl_category = "Tools"

#    def draw(self, context):
#        layout = self.layout
#        layout.operator(RunABlenderPythonCommandForVoiceAttack.bl_idname)



#Registration

addon_keymaps = []

classes = (
RunABlenderPythonCommandForVoiceAttack,
)

def register():
    for cls in classes:
        bpy.utils.register_class(cls)
    
    # add keymap entry
    kcfg = bpy.context.window_manager.keyconfigs.addon
    if kcfg:
        km = kcfg.keymaps.new(name='Window', space_type='EMPTY',region_type ='WINDOW')
        
        kmi = km.keymap_items.new("pythonvoiceattack.code_text_dialog", 'F11','PRESS', ctrl=True, shift=True,alt=True)
        addon_keymaps.append((km, kmi.idname))

def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)
    
    # remove keymap entry
    for km, kmi_idname in addon_keymaps:
        for kmi in km.keymap_items:
            if kmi.idname == kmi_idname:
                km.keymap_items.remove(kmi)
    addon_keymaps.clear()

if __name__ == "__main__":
    register()
