from operator import contains, index
from re import sub
from typing import Dict
import bpy
from mathutils import Vector

def add_math_node():
    bpy.ops.node.add_node(type="ShaderNodeMath", use_transform=True)
    bpy.ops.transform.translate('INVOKE_DEFAULT')
    return

def add_node(query):
    # if (bpy.context.area.type == "NODE_EDITOR"):# ADD THIS
    # Sometimes the __subclasses__ lists are not fully registered.  Don't know why.  Don't have a solution either except to access them manually at first by hand.
    functionNodeClasses = bpy.types.FunctionNode.__subclasses__()
    functionNodeIdentifiers = [node.bl_rna.identifier for node in functionNodeClasses]
    functionNodeNames = [node.bl_rna.name for node in functionNodeClasses]

    shaderNodeClasses = bpy.types.ShaderNode.__subclasses__()
    shaderNodeIdentifiers = [node.bl_rna.identifier for node in shaderNodeClasses]
    shaderNodeNames = [node.bl_rna.name for node in shaderNodeClasses]
    
    geometryNodeClasses = bpy.types.GeometryNode.__subclasses__()
    geometryNodeIdentifiers = [node.bl_rna.identifier for node in geometryNodeClasses]
    geometryNodeNames = [node.bl_rna.name for node in geometryNodeClasses]
    
    compositorNodeClasses = bpy.types.CompositorNode.__subclasses__()
    compositorNodeIdentifiers = [node.bl_rna.identifier for node in compositorNodeClasses]
    compositorNodeNames = [node.bl_rna.name for node in compositorNodeClasses]
    
    # Find the node to create based on area/tree context.
    treeType=bpy.context.space_data.edit_tree.type
    nodeTypeToCreate=""
    if (treeType == "SHADER"):
        # nodeNamePrefix = "ShaderNode"
        search = best_guess_search(shaderNodeNames, query)
        nodeTypeToCreate = shaderNodeIdentifiers[search["index"]]
    elif (treeType == "GEOMETRY"):
        gnIdentifiersList = functionNodeIdentifiers+shaderNodeIdentifiers+geometryNodeIdentifiers
        nodeNamesList = functionNodeNames+shaderNodeNames+geometryNodeNames
        search = best_guess_search(nodeNamesList, query)
        
        for itemIndex in search['indexes']:
            # print(str(i)+ " = " +  gnIdentifiersList[i])
            if("Legacy" not in gnIdentifiersList[itemIndex]):
                nodeTypeToCreate = gnIdentifiersList[itemIndex]
                break
        if(nodeTypeToCreate==None):
            print("Error, no useable node type found!")
        print(f"Node type to create= {nodeTypeToCreate}")
        #     nodeTypeToCreate = gnIdentifiersList[search['index']]
    elif (treeType == "COMPOSITING"):
        # nodeNamePrefix = "CompositorNode"
        search = best_guess_search(compositorNodeNames, query)
        nodeTypeToCreate = compositorNodeIdentifiers[search["index"]]
    
    # nodes = [node.bl_rna.identifier for node in subClasses]
    # # nodes = [node.bl_rna.name for node in subClasses]
    # nodeIdentifiers=[]
    # for node in nodes:
    #     node = node.replace("ShaderNode","")
    #     nodeIdentifiers.append(node)
    # #     list1.append(b.bl_label)
    # print(nodeIdentifiers)
    # x = nodeNamePrefix + best_guess_search(nodeIdentifiers, query)

    #b=bpy.ops.node.add_node(node, use_transform=True)
    bpy.ops.node.add_node(type = nodeTypeToCreate, use_transform=True)
    bpy.ops.transform.translate('INVOKE_DEFAULT')
    return

def get_node_type_from_loose_request():
    print("Not implemented.")
    return 

def set_math_node_operation(operation):
    for node in bpy.context.selected_nodes:
        if (node.type=="MATH"):
            node.operation=best_guess_search(get_enum_values(node, "operation"),operation)["bestGuess"]
        if (node.type=="MIX_RGB"):
            node.blend_type=best_guess_search(get_enum_values(node, "blend_type"),operation)["bestGuess"]

def reset_node_positions():
    source=(0,0)
    for node in bpy.context.selected_nodes:
        if("OUTPUT" in node.type):
            source=node.location
            break
    if(source==(0,0)):
        source=bpy.context.active_node.location
    position=(source.x,source.y)
    for node in bpy.context.selected_nodes:
        node.location+= Vector((-position[0],-position[1]))
    return



#### NON Voice Attack
def best_guess_search(list0, query, currentCaseType="upper"):
    """
    Returns dictionary:\n
    index = int best match index\n
    indexes = [int] All indices in an order based on score.\n
    bestGuess = list0[bestIndex]
    """
    # Format query.
    query = query.lower()
    
    # Get formatted list.
    listF = []
    for item in list0:
        # Un-camel case it.
        if(currentCaseType=="camel"):
            S = list(item)
            for i in range(len(S)):
                if S[i][0].isupper():
                    S[i] = " "+S[i].lower()
            item = ''.join(S)

        item = item.replace("_", " ")
        item = item.strip()
        item = item.lower()
        listF.append(item)

    # Iterate through all items and give points to the best matches.
    # Search method 1: 
    scores=[0]*len(listF)
    foundExactMatch = False
    itemIndex=0
    for item in listF:
        # If the query matches the item exactly.
        if (item==query):
            scores[itemIndex]+=100
        else:
            # If both items contain 'q'
            if (item.count("q")>=1 & query.count("q")>=1):
                scores[itemIndex] += 2  # Give points.
                
            # See how many search-term-snippets are in there:
            #for x in snippets_of_search_term(query):print(x)
            for snippet in snippets_of_search_term(query):
                if (item.count(snippet)>0):# if the search term snippet is in there:
                    scores[itemIndex]+=1# Give points.
                    #print (f"{item} {snippet} ")
        # Show score debug data.
        debug=True
        if(debug):
            if(scores[itemIndex]>0):
                print (f"Score debug: {scores[itemIndex]} {item} ")
        # Next
        itemIndex+=1

    # Use the points given to deduce the best match.
    bestIndex=0
    bestIndexScore=0
    # if(foundExactMatch):
    #     bestIndex = itemIndex
    # else:
    for i in range(len(scores)):
        if (scores[i]>bestIndexScore):
            bestIndex=i
            bestIndexScore=scores[i]
    output={}
    output["bestGuess"] = list0[bestIndex]
    output["index"] = bestIndex
    output["indexes"] = [i for _, i in sorted(zip( scores, range(0, len(list0))))][::-1]# [Slice ::-1] -howDoesThisWork? , reverse = True

    # outInfo.append(bestIndex)
    return output

def snippets_of_search_term(search_term):
    segs=[]
    qchunkSize=9
    while qchunkSize >= 2:
        #print (qchunkSize)
        posMax=len(search_term) -qchunkSize
        pos=0
        while pos <=posMax:
            qchunk=search_term[pos:pos+qchunkSize]
            #print(pos)
            #print(qchunk)
            segs.append(qchunk)
            pos+=1
        qchunkSize-=1
    return segs

def get_enum_values(obj, prop_name):
    return([item.identifier for item in obj.bl_rna.properties[prop_name].enum_items])

#mat = bpy.data.materials['Material']
#nodes = mat.node_tree.nodes

#mNode=nodes.new(type='ShaderNodeMath')
#mNode.operation="GREATER_THAN"

#get_enum_values(mNode,"operation")

